library IEEE;
library unisim;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.image1_pkg.ALL;
use work.wishbone_pkg.ALL;
use UNISIM.VCOMPONENTS.ALL;

entity image1_top is
   generic(g_NUMBER_OF_CHANNELS : NATURAL := 6);
   port (FPGA_CLK_P        : in STD_LOGIC; --Using the 125MHz clock
         FPGA_CLK_N        : in STD_LOGIC;
         led_pw_o          : out STD_LOGIC;
         led_err_o         : out STD_LOGIC;
         led_ttl_o         : out STD_LOGIC;
         led_o_front       : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         led_o_rear        : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_front     : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_front     : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_i_rear      : in  STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         pulse_o_rear      : out STD_LOGIC_VECTOR(g_NUMBER_OF_CHANNELS downto 1);
         inv_i             : in  STD_LOGIC_VECTOR(4 downto 1);
         inv_o             : out STD_LOGIC_VECTOR(4 downto 1);
         --! Lines for the i2c_slave
         SCL_I             : in  STD_LOGIC;
         SCL_O             : out STD_LOGIC;
         SCL_OE            : out STD_LOGIC;
         SDA_I             : in  STD_LOGIC;
         SDA_O             : out STD_LOGIC;
         SDA_OE            : out STD_LOGIC;
         FPGA_GA           : in  STD_LOGIC_VECTOR(4 downto 0);
         FPGA_GAP          : in  STD_LOGIC;
         --! Pins of the SPI interface to write into the Flash memory
         FPGA_PROM_CCLK    : out STD_LOGIC;
         FPGA_PROM_CSO_B_N : out STD_LOGIC;
         FPGA_PROM_DIN     : in  STD_LOGIC;
         FPGA_PROM_MOSI    : out STD_LOGIC;
         --! This LED will show the status of the PLL
         led_link_up_o     : out STD_LOGIC;
         led_pps_o         : out STD_LOGIC;
         led_wr_ok_o       : out STD_LOGIC;
         --! RTM identifiers, should match with the expected values
         --! TODO: add matching
         fpga_o_en         : out STD_LOGIC;
         fpga_o_blo_en     : out STD_LOGIC;
         fpga_o_ttl_en     : out STD_LOGIC;
         fpga_o_inv_en     : out STD_LOGIC;
         level             : in  STD_LOGIC;
         switch_i          : in  STD_LOGIC;   --! General enable
         manual_rst_n_o    : out STD_LOGIC;   --! It allows power sequencing of the
                                              --! 24V rail after a security given
                                              --! delay
         FPGA_RTMM         : in STD_LOGIC_VECTOR(2 downto 0);
         FPGA_RTMP         : in STD_LOGIC_VECTOR(2 downto 0));
end image1_top;


architecture Behavioral of image1_top is

   type t_clocks is
      record
         PLL_IN      : STD_LOGIC; --! 125MHz  Input clock
         PLL_FB_IN   : STD_LOGIC;
         PLL_FB_OUT  : STD_LOGIC;
         PLL_SYS_A   : STD_LOGIC; --! 50MHz   Wishbone clock
         PLL_SYS_B   : STD_LOGIC; --! 200MHz  Pulse sampling
         SYS_A       : STD_LOGIC;
         SYS_B       : STD_LOGIC;
      end record;

   type t_rst is
      record
         PLL    : STD_LOGIC_VECTOR(c_RST_PLL_CLKS - 1 downto 0);
         SYS_A  : STD_LOGIC_VECTOR(c_RST_A_CLKS   - 1 downto 0);
         SYS_B  : STD_LOGIC_VECTOR(c_RST_B_CLKS   - 1 downto 0);
      end record;

   signal s_clk                : t_clocks;
   signal s_rst                : t_rst := (PLL   => (others => '1'),
                                           SYS_A => (others => '1'),
                                           SYS_B => (others => '1'));

   signal s_locked             : STD_LOGIC;

   signal s_ok_RTMM            : STD_LOGIC;
   signal s_ok_RTMP            : STD_LOGIC;

   signal s_led_err_o          : STD_LOGIC;
   signal s_led_err_basic_trig : STD_LOGIC;

   signal s_slave_i  : t_wishbone_slave_in_array   (c_NUM_MASTERS - 1 downto 0);
   signal s_slave_o  : t_wishbone_slave_out_array  (c_NUM_MASTERS - 1 downto 0);
   signal s_master_i : t_wishbone_master_in_array  (c_NUM_SLAVES  - 1 downto 0);
   signal s_master_o : t_wishbone_master_out_array (c_NUM_SLAVES  - 1 downto 0);

   signal s_i2c_addr : STD_LOGIC_VECTOR(6 downto 0);

--   signal s_check_cfg : BOOLEAN;
begin

--   s_check_cfg     <= check_sys_cfg;

   s_i2c_addr      <= f_RENESAS_I2C_ADDRESSING(UNSIGNED(FPGA_GA));

   s_led_err_o     <= '1' when s_ok_RTMM = '1' AND s_ok_RTMP = '1'
                          else '0';
   led_err_o       <= s_led_err_o OR s_led_err_basic_trig;

   inst_125m_IBUFGDS : IBUFGDS
   generic map (DIFF_TERM    => TRUE,  IBUF_LOW_PWR => TRUE)
   port map (O  => s_clk.PLL_IN, I  => FPGA_CLK_P,
                                 IB => FPGA_CLK_N);

   s_clk_fb_bufg   : BUFG
     port map (O => s_clk.PLL_FB_IN, I => s_clk.PLL_FB_OUT);

   -- set up the fabric PLL_BASE to drive the BUFPLL
   pll_base_inst : PLL_BASE
     generic map (
       BANDWIDTH          => "OPTIMIZED",
       CLK_FEEDBACK       => "CLKFBOUT",
       COMPENSATION       => "SYSTEM_SYNCHRONOUS",
       DIVCLK_DIVIDE      => 1,  CLKFBOUT_MULT  => c_CLKFBOUT_MULT,
                                 CLKFBOUT_PHASE => 0.000,
       CLKOUT0_DIVIDE     => c_CLKOUTA_DIVIDE, CLKOUT0_PHASE      => 0.000,
                                               CLKOUT0_DUTY_CYCLE => 0.500,
       CLKOUT1_DIVIDE     => c_CLKOUTB_DIVIDE, CLKOUT1_PHASE      => 0.000,
                                               CLKOUT1_DUTY_CYCLE => 0.500,
       CLKIN_PERIOD       => c_CLKIN_PERIOD,
       REF_JITTER         => 0.010)
     port map (
       -- Output clocks
       CLKFBOUT => s_clk.PLL_FB_OUT,
       CLKOUT0  => s_clk.PLL_SYS_A,
       CLKOUT1  => s_clk.PLL_SYS_B,
       CLKOUT2  => open,      CLKOUT3  => open,
       CLKOUT4  => open,      CLKOUT5  => open,
       -- Status and control signals
       LOCKED   => s_locked,  RST      => s_rst.PLL(c_RST_PLL_CLKS - 1),
       -- Input clock control
       CLKFBIN  => s_clk.PLL_FB_IN,
       CLKIN    => s_clk.PLL_IN);

   s_clk_50M_bufg : BUFG
     port map (O => s_clk.SYS_A,     I => s_clk.PLL_SYS_A);

   s_clk_200M_bufg : BUFG
     port map (O => s_clk.SYS_B,     I => s_clk.PLL_SYS_B);

   --! @brief            Reset chain for PLL
   --! @param s_clk.PLL  Wishbone clock
   p_reset_PLL_chain : process(s_clk.PLL_IN) is
   begin
      if rising_edge(s_clk.PLL_IN) then
            s_rst.PLL <= (others => '1');

            s_rst.PLL(0) <= '0';
            for i in 1 to c_RST_PLL_CLKS - 1 loop
               s_rst.PLL(i) <= s_rst.PLL(i-1);
            end loop;

            if s_rst.PLL(c_RST_PLL_CLKS - 1) = '1' then
            end if;

         end if;
   end process p_reset_PLL_chain;


   --! @brief              Reset chain
   --! @param s_clk.SYS_A  Wishbone clock
   p_reset_A_chain : process(s_clk.SYS_A) is
   begin
      if rising_edge(s_clk.SYS_A) then
         if s_locked = '0' then
            s_rst.SYS_A    <= (others => '1');
         else
            s_rst.SYS_A(0) <= '0';
            for i in 1 to c_RST_A_CLKS - 1 loop
               s_rst.SYS_A(i) <= s_rst.SYS_A(i-1);
            end loop;

            if s_rst.SYS_A(c_RST_A_CLKS - 1) = '1' then
            end if;

         end if;
      end if;
   end process p_reset_A_chain;

   --! @brief              Reset chain
   --! @param s_clk.SYS_B  System of clock domain B
   p_reset_B_chain : process(s_clk.SYS_B) is
   begin
      if rising_edge(s_clk.SYS_B) then
         if s_locked = '0' then
            s_rst.SYS_B    <= (others => '1');
         else
            s_rst.SYS_B(0) <= '0';
            for i in 1 to c_RST_B_CLKS - 1 loop
               s_rst.SYS_B(i) <= s_rst.SYS_B(i-1);
            end loop;

            if s_rst.SYS_B(c_RST_B_CLKS - 1) = '1' then
            end if;

         end if;
      end if;
   end process p_reset_B_chain;


   inst_basic_trigger: basic_trigger_top
   generic map(g_NUMBER_OF_CHANNELS   => c_NUMBER_OF_CHANNELS,
               g_CLK_PERIOD           => c_CLKB_PERIOD,
               g_OUTPUT_PULSE_LENGTH  => c_OUTPUT_PULSE_LENGTH,
               g_LED_BLINKING_LENGTH  => c_LED_BLINKING_LENGTH)
   port map(clk_i          => s_clk.SYS_B,
            rst_i          => s_rst.SYS_B(c_RST_B_CLKS - 1),
            led_pw_o       => led_pw_o,
            led_err_o      => s_led_err_basic_trig,
            led_ttl_o      => led_ttl_o,
            fpga_o_en      => fpga_o_en,
            fpga_o_ttl_en  => fpga_o_ttl_en,
            fpga_o_inv_en  => fpga_o_inv_en,
            fpga_o_blo_en  => fpga_o_blo_en,
            level          => level,
            switch_i       => switch_i,
            manual_rst_n_o => manual_rst_n_o,
            pulse_i_front  => pulse_i_front,
            pulse_o_front  => pulse_o_front,
            pulse_i_rear   => pulse_i_rear,
            pulse_o_rear   => pulse_o_rear,
            led_o_front    => led_o_front,
            led_o_rear     => led_o_rear,
            led_link_up_o  => led_link_up_o,
            led_pps_o      => led_pps_o,
            led_wr_ok_o    => led_wr_ok_o,
            inv_i          => inv_i,
            inv_o          => inv_o);

   --! We are always the slave so we disable writes into SCL pin
   SCL_OE <= '0';

   -- s_slave_o(c_MASTER_I2C_SLAVE).stall;
   -- s_slave_o(c_MASTER_I2C_SLAVE).int;
   -- s_master_i(c_SLAVE_I2C_SLAVE).stall,
   -- s_master_i(c_SLAVE_I2C_SLAVE).int,

   s_slave_i(c_MASTER_I2C_SLAVE).adr(31 downto 16) <= (others => '0');

   inst_i2c_slave: i2c_slave_top
   port map(sda_oen           => SDA_OE,
            sda_i             => SDA_I,
            sda_o             => SDA_O,
            scl_oen           => open,
            scl_i             => SCL_I,
            scl_o             => SCL_O,
            wb_clk            => s_clk.SYS_A,
            wb_rst_i          => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_master_stb_o   => s_slave_i(c_MASTER_I2C_SLAVE).stb,
            wb_master_cyc_o   => s_slave_i(c_MASTER_I2C_SLAVE).cyc,
            wb_master_sel_o   => s_slave_i(c_MASTER_I2C_SLAVE).sel,
            wb_master_we_o    => s_slave_i(c_MASTER_I2C_SLAVE).we,
            wb_master_data_i  => s_slave_o(c_MASTER_I2C_SLAVE).dat,
            wb_master_data_o  => s_slave_i(c_MASTER_I2C_SLAVE).dat,
            wb_master_addr_o  => s_slave_i(c_MASTER_I2C_SLAVE).adr(15 downto 0),
            wb_master_ack_i   => s_slave_o(c_MASTER_I2C_SLAVE).ack,
            wb_master_rty_i   => s_slave_o(c_MASTER_I2C_SLAVE).rty,
            wb_master_err_i   => s_slave_o(c_MASTER_I2C_SLAVE).err,
            wb_slave_stb_i    => s_master_o(c_SLAVE_I2C_SLAVE).stb,
            wb_slave_cyc_i    => s_master_o(c_SLAVE_I2C_SLAVE).cyc,
            wb_slave_sel_i    => s_master_o(c_SLAVE_I2C_SLAVE).sel,
            wb_slave_we_i     => s_master_o(c_SLAVE_I2C_SLAVE).we,
            wb_slave_data_i   => s_master_o(c_SLAVE_I2C_SLAVE).dat,
            wb_slave_data_o   => s_master_i(c_SLAVE_I2C_SLAVE).dat,
            wb_slave_addr_i   => s_master_o(c_SLAVE_I2C_SLAVE).adr( 5 downto 2),
            wb_slave_ack_o    => s_master_i(c_SLAVE_I2C_SLAVE).ack,
            wb_slave_rty_o    => s_master_i(c_SLAVE_I2C_SLAVE).rty,
            wb_slave_err_o    => s_master_i(c_SLAVE_I2C_SLAVE).err,
            pf_wb_addr_o      => open,
            rd_done_o         => open,
            wr_done_o         => open,
            i2c_addr_i        => s_i2c_addr);



   inst_m25p32: m25p32_top
   port map(wb_rst_i          => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_clk            => s_clk.SYS_A,
            wb_we_i           => s_master_o(c_SLAVE_M25P32).we,
            wb_stb_i          => s_master_o(c_SLAVE_M25P32).stb,
            wb_cyc_i          => s_master_o(c_SLAVE_M25P32).cyc,
            wb_sel_i          => s_master_o(c_SLAVE_M25P32).sel,
            wb_data_i         => s_master_o(c_SLAVE_M25P32).dat,
            wb_data_o         => s_master_i(c_SLAVE_M25P32).dat,
            wb_addr_i         => s_master_o(c_SLAVE_M25P32).adr( 8 downto 2),
            wb_ack_o          => s_master_i(c_SLAVE_M25P32).ack,
            wb_rty_o          => s_master_i(c_SLAVE_M25P32).rty,
            wb_err_o          => s_master_i(c_SLAVE_M25P32).err,
            miso_word_rcv     => open,
            op_finished_o     => open,
            prom_mosi_o       => FPGA_PROM_MOSI,
            prom_cclk_o       => FPGA_PROM_CCLK,
            prom_cs0_b_n_o    => FPGA_PROM_CSO_B_N,
            prom_din_i        => FPGA_PROM_DIN);


   inst_multiboot: multiboot_top
   port map(wb_rst_i  => s_rst.SYS_A(c_RST_A_CLKS - 1),
            wb_clk    => s_clk.SYS_A,
            wb_we_i   => s_master_o(c_SLAVE_MULTIBOOT).we,
            wb_stb_i  => s_master_o(c_SLAVE_MULTIBOOT).stb,
            wb_cyc_i  => s_master_o(c_SLAVE_MULTIBOOT).cyc,
            wb_sel_i  => s_master_o(c_SLAVE_MULTIBOOT).sel,
            wb_data_i => s_master_o(c_SLAVE_MULTIBOOT).dat,
            wb_data_o => s_master_i(c_SLAVE_MULTIBOOT).dat,
            wb_addr_i => s_master_o(c_SLAVE_MULTIBOOT).adr( 5 downto 2),
            wb_ack_o  => s_master_i(c_SLAVE_MULTIBOOT).ack,
            wb_rty_o  => s_master_i(c_SLAVE_MULTIBOOT).rty,
            wb_err_o  => s_master_i(c_SLAVE_MULTIBOOT).err);


   s_master_i(c_SLAVE_I2C_SLAVE).stall  <= '0';
   s_master_i(c_SLAVE_I2C_SLAVE).int    <= '0';
   s_master_i(c_SLAVE_M25P32).stall     <= '0';
   s_master_i(c_SLAVE_M25P32).int       <= '0';
   s_master_i(c_SLAVE_MULTIBOOT).stall  <= '0';
   s_master_i(c_SLAVE_MULTIBOOT).int    <= '0';

   inst_wb_crossbar: xwb_crossbar
   generic map(g_num_masters => c_NUM_MASTERS,
               g_num_slaves  => c_NUM_SLAVES,
               g_registered  => false,
               -- Address of the slaves connected
               --! It should be noted that the default address length is 32
               --! In our project only 16 bits are addressable
               g_address     => c_addresses,
               g_mask        => c_masks)
   port map(clk_sys_i => s_clk.SYS_A,
            rst_n_i   => s_rst.SYS_A(c_RST_A_CLKS - 1),
            slave_i   => s_slave_i ,
            slave_o   => s_slave_o ,
            master_i  => s_master_i,
            master_o  => s_master_o);

   inst_rtm_detector: rtm_detector
   port map(RTMM_i     => FPGA_RTMM,
            RTMP_i     => FPGA_RTMP,
            ok_RTMM_o  => s_ok_RTMM,
            ok_RTMP_o  => s_ok_RTMP);


end Behavioral;
