--==============================================================================
-- CERN (BE-CO-HT)
-- Pulse generator with trigger
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-03-01
--
-- version: 1.0
--
-- description:
--    This module generates a variable-width pulse. The width is set using the
--    g_pulse_width generic, given in number of clk_i cycles. Assuming a clk_i
--    period of 8ns, the output pulse width is by default 8*16=128ns.
--
--    The pulse_o signal is generated by multiplexing the input trigger and
--    the internally-generated pulse. When the trigger is detected, it is
--    immediately passed to the output in order to avoid jitter. 
--
--    The trigger then passes through a variable-length glitch filter consisting 
--    of N flip-flops, where N is set via g_glitch_flit_len. Once the trigger 
--    has settled to '1' and the glitch filter is all ones, the internal pulse 
--    signal takes over, extending the input pulse up to g_pulse_width.
--
--    Selection between trigger input and internal pulse signal routing to the
--    output is done by an internal pulse rejection signal, which is set once
--    the internal pulse generation mechanism has started, and cleared when
--    the trigger input has settled to zero. This signal assures that trigger
--    signals no greater than the desired pulse width appear at the input, thus
--    safeguarding the output transformers on the blocking pulse.
--
-- dependencies:
--    none
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-03-01   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ctb_pulse_gen is
  generic 
  (
    -- Pulse width, in number of clk_i cycles
    g_pulse_width     : natural := 15;

    -- Glitch filter length:
    -- g_glitch_filt_len=1 => trigger width should be > 1 clk_i cycle
    -- g_glitch_filt_len=2 => trigger width should be > 2 clk_i cycles
    -- etc.
    g_glitch_filt_len : natural := 4

  );
  port
  (
    clk_i        : in  std_logic;
    rst_n_i      : in  std_logic;
    
    -- Pulse type input
    -- '1' - TYPE 1 (glitch-sensitive, without output jitter)
    -- '0' - TYPE 2 (glitch-insensitive, with output jitter)
    pulse_type_i : in  std_logic;

    -- Enable input, pulse generation is enabled when '1'
    en_i         : in  std_logic;

    -- Trigger input, has to be '1' to assure pulse output with delay no greater
    -- than internal gate delays.
    trig_i       : in  std_logic;
    
    -- Pulse output, active-high
    -- latency:
    --    TYPE 1 pulse: none
    --    TYPE 2 pulse: g_glitch_filt_len+3 clk_i cycles
    pulse_o      : out std_logic
  );
end entity ctb_pulse_gen;


architecture behav of ctb_pulse_gen is

  --============================================================================
  -- Type declarations
  --============================================================================
  type t_state is (ST_IDLE, ST_PULSE_TYPE1, ST_PULSE_TYPE2);

  --============================================================================
  -- Function and procedure declarations
  --============================================================================
  function f_log2_size (A : natural) return natural is
  begin
    for I in 1 to 64 loop               -- Works for up to 64 bits
      if (2**I >= A) then
        return(I);
      end if;
    end loop;
    return(63);
  end function f_log2_size;

  --============================================================================
  -- Component declarations
  --============================================================================
  component glitch_filt is
    generic
    (
    -- Length of glitch filter:
    -- g_len = 1 => data width should be > 1 clk_i cycle
    -- g_len = 2 => data width should be > 2 clk_i cycle
    -- etc.
      g_len : natural := 4
    );
    port
    (
      clk_i   : in  std_logic;
      rst_n_i : in  std_logic;

    -- Data input
      dat_i   : in  std_logic;

    -- Data output
    -- latency: g_len+1 clk_i cycles
      dat_o   : out std_logic
    );
  end component glitch_filt;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Deglitched trigger
  signal trig_degl      : std_logic;
  signal trig_degl_d0   : std_logic;

  -- Pulse length counter
  signal width_cnt      : unsigned(f_log2_size(g_pulse_width)-1 downto 0);
  
  -- Pulse-specific signals
  signal pulse_type1    : std_logic;
  signal pulse_type1_d0 : std_logic;
  signal pulse_type1_d1 : std_logic;
  signal pulse_type1_d2 : std_logic;
  signal pulse_rst      : std_logic;
  
  signal pulse_type2    : std_logic;

  -- FSM signal
  signal state          : t_state;
  
--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- Output logic
  --============================================================================
  pulse_o <= pulse_type1 when (pulse_type_i = '1') else
             pulse_type2;

  --============================================================================
  -- Pulse generation logic
  --============================================================================
  -- Generate the pulse on rising edge of trig_i
  p_pulse_type1: process(pulse_rst, trig_i)
  begin
    if (pulse_rst = '1') then
      pulse_type1 <= '0';
    elsif rising_edge(trig_i) then
      if (en_i = '1') then
        pulse_type1 <= '1';
      end if;
    end if;
  end process p_pulse_type1;

  -- and synchronize it in clk_i domain
  p_sync_pulse_type1: process (clk_i) is
  begin
    if rising_edge(clk_i) then
      if (rst_n_i = '0') then
        pulse_type1_d0 <= '0';
        pulse_type1_d1 <= '0';
        pulse_type1_d2 <= '0';
      elsif (en_i = '1') then
        pulse_type1_d0 <= pulse_type1;
        pulse_type1_d1 <= pulse_type1_d0;
        pulse_type1_d2 <= pulse_type1_d1;
      end if;
    end if;
  end process p_sync_pulse_type1;
  
  -- Type 2 pulse is generated when the FSM is counting to the pulse width
  p_pulse_type2: process(clk_i)
  begin
    if rising_edge(clk_i) then
      if (rst_n_i = '0') then
        pulse_type2 <= '0';
      elsif (en_i = '1') then
        pulse_type2 <= '0';
        if (state = ST_PULSE_TYPE2) then
          pulse_type2 <= '1';
        end if;
      end if;
    end if;
  end process p_pulse_type2;
  
  --============================================================================
  -- Glitch filtration logic
  --============================================================================
  cmp_glitch_filt: glitch_filt
    generic map
    (
      g_len => g_glitch_filt_len
    )
    port map
    (
      clk_i   => clk_i,
      rst_n_i => rst_n_i,
      dat_i   => trig_i,
      dat_o   => trig_degl
    );

  --============================================================================
  -- Pulse width adjustment logic
  --============================================================================
  p_pulse_width: process(clk_i)
  begin
    if rising_edge(clk_i) then
      if (rst_n_i = '0') then
        state        <= ST_IDLE;
        pulse_rst    <= '1';
        width_cnt    <= (others => '0');
        trig_degl_d0 <= '0';
      elsif (en_i = '1') then
        -- Deglitched trigger delay
        trig_degl_d0 <= trig_degl;

        -- State machine
        case state is
          ---------------------------------------------------------------------
          -- ST_IDLE
          ---------------------------------------------------------------------
          -- Clear all values and go to type 1 or type 2 pulse generation
          -- when the appropriate input arrives
          ---------------------------------------------------------------------
          when ST_IDLE =>
            width_cnt <= (others => '0');
            pulse_rst <= '0';
            if (pulse_type_i = '1') then
              if (pulse_type1_d1 = '1') and (pulse_type1_d2 = '0') then
                state <= ST_PULSE_TYPE1;
              end if;
            else
              if (trig_degl = '1') and (trig_degl_d0 = '0') then
                state <= ST_PULSE_TYPE2;
              end if;
            end if;

          ---------------------------------------------------------------------
          -- ST_PULSE_TYPE1
          ---------------------------------------------------------------------
          -- Increment width counter to maximum value, then generate reset 
          -- signal for type 1 pulse.
          --
          -- Max value: g_pulse_width-5 due to:
          -- 1. width_cnt starts from 0 => g_pulse_width-1
          -- 2. three cycle delay from FFs in gc_sync_ffs
          -- 3. one clock cycle delay for switching from ST_IDLE to ST_PULSE_TYPE1 
          ---------------------------------------------------------------------
          when ST_PULSE_TYPE1 =>
            width_cnt <= width_cnt + 1;
            if (width_cnt = g_pulse_width - 5) then
              pulse_rst <= '1';
              state     <= ST_IDLE;
            end if;

          ---------------------------------------------------------------------
          -- ST_PULSE_TYPE2
          ---------------------------------------------------------------------
          -- Increment counter to max value and generate type 2 pulse while
          -- incrementing.
          --
          -- Max value: g_pulse_width, because we start outputting the 
          -- pulse while already in this state, on the cycle when
          -- width_cnt = 1
          ---------------------------------------------------------------------
          when ST_PULSE_TYPE2 =>
            width_cnt <= width_cnt + 1;
            if (width_cnt = g_pulse_width - 1) then
              pulse_rst <= '1';
              state     <= ST_IDLE;
            end if;
            
          when others =>
            state <= ST_IDLE;

        end case;
      end if;
    end if;
  end process p_pulse_width;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
