-- TestBench Template
library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use STD.TEXTIO.ALL;

--! Library added to use Signal Spy features
library modelsim_lib;
use modelsim_lib.util.ALL;

use work.multiboot_pkg.ALL;
use work.multiboot_pkg_tb.ALL;
use work.wishbone_driver_pkg.ALL;


entity multiboot_tb is
   end multiboot_tb;

architecture behavior of multiboot_tb is

   signal wb_rst_i  : STD_LOGIC;
   signal wb_clk    : STD_LOGIC;
   signal wb_stb_i  : STD_LOGIC;
   signal wb_cyc_i  : STD_LOGIC;
   signal wb_sel_i  : STD_LOGIC_VECTOR(3 downto 0);
   signal wb_we_i   : STD_LOGIC;
   signal wb_data_i : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_data_o : STD_LOGIC_VECTOR(31 downto 0);
   signal wb_addr_i : STD_LOGIC_VECTOR(3  downto 0);
   signal wb_ack_o  : STD_LOGIC;
   signal wb_rty_o  : STD_LOGIC;
   signal wb_err_o  : STD_LOGIC;

   signal s_wb_driver_data_i : STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
   signal s_wb_driver_data_o : STD_LOGIC_VECTOR(31 downto 0);
   signal s_wb_driver_addr   : STD_LOGIC_VECTOR(3  downto 0) := (others => '0');
   signal s_write_i          : STD_LOGIC := '0';
   signal s_write_done_o     : STD_LOGIC;
   signal s_read_i           : STD_LOGIC := '0';
   signal s_read_done_o      : STD_LOGIC;

begin

   wb_driver: wishbone_driver
   generic map(g_WB_DATA_O_LENGTH => 32,
               g_WB_DATA_I_LENGTH => 32,
               g_WB_ADDR_LENGTH   => 4,
               g_WB_CLK_PERIOD    => c_WB_CLK_PERIOD)
   port map(wb_clk_i        => wb_clk,
            wb_rst_i        => wb_rst_i,

            wb_stb_o        => wb_stb_i,
            wb_cyc_o        => wb_cyc_i,
            wb_sel_o        => wb_sel_i,
            wb_we_o         => wb_we_i,
            wb_data_i       => wb_data_o,
            wb_data_o       => wb_data_i,
            wb_addr_o       => wb_addr_i,
            wb_ack_i        => wb_ack_o,
            wb_rty_i        => wb_rty_o,
            wb_err_i        => wb_err_o,

            data_i          => s_wb_driver_data_i,
            data_o          => s_wb_driver_data_o,
            addr_i          => s_wb_driver_addr,

            write_i         => s_write_i,
            write_done_o    => s_write_done_o,
            read_i          => s_read_i,
            read_done_o     => s_read_done_o);

   uut: multiboot_top
   port map(wb_rst_i  =>  wb_rst_i,
            wb_clk    =>  wb_clk,
            wb_we_i   =>  wb_we_i,
            wb_stb_i  =>  wb_stb_i,
            wb_cyc_i  =>  wb_cyc_i,
            wb_sel_i  =>  wb_sel_i,
            wb_data_i =>  wb_data_i,
            wb_data_o =>  wb_data_o,
            wb_addr_i =>  wb_addr_i,
            wb_ack_o  =>  wb_ack_o,
            wb_rty_o  =>  wb_rty_o,
            wb_err_o  =>  wb_err_o);

   --! @brief Process to bypass internal signals requiered for correctly
   --!        checking
   p_sig_spy : process is
   begin
        init_signal_spy("/uut/multiboot_core_inst/s_ICAP_fsm",
                      "s_ICAP_fsm", 1);
        init_signal_spy("/uut/multiboot_core_inst/s_multiboot_fsm",
                      "s_multiboot_fsm", 1);
        init_signal_spy("/uut/multiboot_core_inst/s_readback_fsm",
                      "s_readback_fsm", 1);
        wait;
   end process p_sig_spy;


   --! @brief Main clock generation process
   wb_clk_process: process
   begin
      wb_clk <= '1';
        wait for c_WB_CLK_PERIOD/2;
      wb_clk <= '0';
        wait for c_WB_CLK_PERIOD/2;
   end process;

   --! @brief Testbench process
   tb : process

      variable v_header_line_buffer  : LINE;

      procedure rst is
      begin
           wait until rising_edge(wb_clk);
         wb_rst_i  <= '1';
           wait for c_WB_CLK_PERIOD*4;
           wait until rising_edge(wb_clk);
         wb_rst_i  <= '0';
           wait until rising_edge(wb_clk);
      end rst;

      --! @brief  Procedure to place a read to memory via wishbone interface
      --! @param addr  Wishbone address to write to
      --! @param data  Wishbone data to write
      procedure write_wb_addr(addr : STD_LOGIC_VECTOR(3  downto 0);
                              data : STD_LOGIC_VECTOR(31 downto 0)) is
      begin
         s_wb_driver_addr   <= addr;
         s_wb_driver_data_i <= data;
         s_write_i          <= '1';
           wait until rising_edge(wb_clk);
           wait until s_write_done_o = '1';
         s_wb_driver_addr   <= (others => '0');
         s_wb_driver_data_i <= (others => '0');
         s_write_i          <= '0';
           wait until rising_edge(wb_clk);
      end procedure;

      --! @brief       Procedure to place a read to memory via wishbone
      --!              interface
      --! @param addr  Wishbone address to read from
      procedure read_wb_addr(addr : STD_LOGIC_VECTOR(3  downto 0)) is
      begin
         s_wb_driver_addr   <= addr;
         s_write_i          <= '0';
           wait until rising_edge(wb_clk);
           wait until s_read_done_o = '1';
         --! At that point in time we have correct data in s_wb_driver_data_o
         --! signal.
         s_wb_driver_addr   <= (others => '0');
         s_write_i          <= '0';
           wait until rising_edge(wb_clk);
      end procedure;


      --! @brief            Wrapper to write CTR0 via wishbone
      --! @param operation  operation to be performed and checked
      procedure write_CTR0(operation: UNSIGNED(3  downto 0)) is
         variable v_CTR0      : r_CTR0 := c_CTR0_default;
         variable v_CTR0_slv  : STD_LOGIC_VECTOR(r_CTR0'a_length - 1 downto 0);
      begin
         v_CTR0.OP   := operation;
         v_CTR0.PEN  := '1';
         v_CTR0.x    := (others => '0');
         v_CTR0_slv  := f_STD_LOGIC_VECTOR(v_CTR0);
         write_wb_addr(STD_LOGIC_VECTOR(CTR0_addr), X"000000"&v_CTR0_slv);
      end procedure;

--      procedure checker_and_logger() is
--      begin
--      end procedure;

      --! @brief            procedure to check ICAP's readback subfsm
      --! @param operation  operation to be performed and checked
      procedure check_readback_fsm(operation: UNSIGNED (3 downto 0)) is
      begin
      end procedure;

      --! @brief            procedure to check ICAP's multiboot subfsm
      --! @param operation  operation to be performed and checked
      procedure check_multiboot_fsm(operation: UNSIGNED (3 downto 0)) is
      begin
      end procedure;

      --! @brief            procedure to check ICAP fsm
      --! @param operation  operation to be performed and checked
      procedure check_ICAP_fsm(operation: UNSIGNED (3 downto 0)) is
      begin
         wait until rising_edge(wb_clk);
         wait until rising_edge(wb_clk);
         wait until rising_edge(wb_clk);
         wait until rising_edge(wb_clk);
         wait until rising_edge(wb_clk);
         wait until rising_edge(wb_clk);
         case operation is
            when OP_NONE          =>
            when OP_FULLMULTIBOOT =>
               check_multiboot_fsm(operation);
            when OP_WR_MBA        =>
               check_multiboot_fsm(operation);
            when OP_WR_GBA        =>
               check_multiboot_fsm(operation);
            when OP_RD_MBA        =>
               check_readback_fsm(operation);
            when OP_RD_GBA        =>
               check_readback_fsm(operation);
            when OP_RD_STAT       =>
               check_readback_fsm(operation);
            when others           =>
               --! Bad directed test
               null;
         end case;
      end procedure;

      --! @brief            procedure to check the correctness of an operation
      --! @param operation  operation to be performed and checked
      procedure write_check_operation(operation: UNSIGNED( 3 downto 0)) is
      begin
         write_CTR0(operation);
         check_ICAP_fsm(operation);
      end procedure;

   begin
      file_open(file_handler, c_LOG_PATH, WRITE_MODE);
      write(v_header_line_buffer, string'("Test ID"&sep&"Status"&sep&"Description"));
      writeline(file_handler, v_header_line_buffer);

      rst;

        wait for c_WB_CLK_PERIOD*35000;
        wait until rising_edge(wb_clk);
--      --! Testbench procedure
--      --! 1.- OP_NONE          operation
--      write_check_operation(OP_NONE);
      --! 2.- OP_FULLMULTIBOOT operation
      write_check_operation(OP_FULLMULTIBOOT);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 3.- OP_WR_MBA        operation
      write_check_operation(OP_WR_MBA);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 4.- OP_WR_GBA        operation
      write_check_operation(OP_WR_GBA);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 5.- OP_CMD_IPROG     operation
      write_check_operation(OP_CMD_IPROG);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 6.- OP_RD_MBA        operation
      write_check_operation(OP_RD_MBA);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 7.- OP_RD_GBA        operation
      write_check_operation(OP_RD_GBA);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      --! 8.- OP_RD_STAT       operation
      write_check_operation(OP_RD_STAT);
        wait for c_WB_CLK_PERIOD*100;
        wait until rising_edge(wb_clk);
      wait for c_WB_CLK_PERIOD*2000;
      wait until rising_edge(wb_clk);
      assert false report "No error. Simulation ends." severity failure;
   end process tb;

end;
