--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Top level entity of CONV-TTL-BLO V1
-- http://www.ohwr.org/projects/conv-trig-blo
--------------------------------------------------------------------------------
--
-- unit name: conv_ttl_blo_v2.vhd
--
-- author: Theodor-Adrian Stana (t.stana@cern.ch)
--
-- version: 1.0
--
-- description: Top entity of CONV-TTL-BLO V1
--
-- dependencies:
--
-- references:
--   [1] ELMA, Access to board data using SNMP and I2C
--       http://www.ohwr.org/documents/227
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------
library IEEE;
library unisim;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use UNISIM.VCOMPONENTS.ALL;

use work.bicolor_led_ctrl_pkg.all;
use work.wishbone_pkg.all;

entity conv_ttl_blo_v2 is
  generic
  (
    g_nr_ttl_chan : natural := 6;
    g_nr_inv_chan : natural := 4
  );
  port 
  (
--    clk20_vcxo_i           : in  std_logic;
    fpga_clk_p_i           : in  std_logic; --Using the 125MHz clock
    fpga_clk_n_i           : in  std_logic;
    
    -- LEDs
    led_ctrl0_o            : out std_logic;
    led_ctrl0_oen_o        : out std_logic;
    led_ctrl1_o            : out std_logic;
    led_ctrl1_oen_o        : out std_logic;
    led_multicast_2_0_o    : out std_logic;
    led_multicast_3_1_o    : out std_logic;
    led_wr_gmt_ttl_ttln_o  : out std_logic;
    led_wr_link_syserror_o : out std_logic;
    led_wr_ok_syspw_o      : out std_logic;
    led_wr_ownaddr_i2c_o   : out std_logic;
    
    -- I/Os for pulses
    pulse_front_led_n_o    : out std_logic_vector(g_nr_ttl_chan downto 1);
    pulse_rear_led_n_o     : out std_logic_vector(g_nr_ttl_chan downto 1);
    fpga_input_ttl_n_i     : in  std_logic_vector(g_nr_ttl_chan downto 1);
    fpga_out_ttl_o         : out std_logic_vector(g_nr_ttl_chan downto 1);
    fpga_blo_in_i          : in  std_logic_vector(g_nr_ttl_chan downto 1);
    fpga_trig_blo_o        : out std_logic_vector(g_nr_ttl_chan downto 1);
    inv_in_n_i             : in  std_logic_vector(g_nr_inv_chan downto 1);
    inv_out_o              : out std_logic_vector(g_nr_inv_chan downto 1);
    
    -- Output enable lines
    fpga_oe_o              : out std_logic;
    fpga_blo_oe_o          : out std_logic;
    fpga_trig_ttl_oe_o     : out std_logic;
    fpga_inv_oe_o          : out std_logic;
    
    --TTL/INV_TTL_N
    ttl_switch_n_i         : in  std_logic;
    
    extra_switch_n_i       : in  std_logic_vector(7 downto 1);
    
    -- Lines for the i2c_slave
    scl_i                  : in  std_logic;
    scl_o                  : out std_logic;
    scl_oe_o               : out std_logic;
    sda_i                  : in  std_logic;
    sda_o                  : out std_logic;
    sda_oe_o               : out std_logic;
    fpga_ga_i              : in  std_logic_vector(4 downto 0);
    fpga_gap_i             : in  std_logic;

    -- It allows power sequencing of the 24V rail after a security delay
    mr_n_o                 : out std_logic;

    -- RTM identifiers, should match with the expected values
    -- TODO: add matching
    fpga_rtmm_n_i        : in std_logic_vector(2 downto 0);
    fpga_rtmp_n_i        : in std_logic_vector(2 downto 0)
  );
end conv_ttl_blo_v2;


architecture behav of conv_ttl_blo_v2 is

  --============================================================================
  -- Type declarations
  --============================================================================
  type t_ttlbar_nosig_cnt is array (1 to g_nr_ttl_chan) of unsigned(13 downto 0);

  --============================================================================
  -- Constant declarations
  --============================================================================
  -- Firmware version
  constant c_fwvers          : std_logic_vector(15 downto 0) := x"0101";
  -- Number of Wishbone masters and slaves, for wb_crossbar
  constant c_nr_masters      : natural :=  1;
  constant c_nr_slaves       : natural :=  1;

  -----------------------------------------
  -- Memory map
  -- * all registers are word-addressable
  -- * all registers are word-aligned
  -----------------------------------------
  --    CONV_REGS       [000-040]
  -----------------------------------------
  -- slave order definitions
  constant c_slv_conv_regs : natural := 0;

  -- base address definitions
  constant c_addr_conv_regs : t_wishbone_address := x"00000000";

  -- address mask definitions
  constant c_mask_conv_regs : t_wishbone_address := x"00000FC0";

  -- addresses constant for Wishbone crossbar
  constant c_addresses : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                  := (
                       c_slv_conv_regs => c_addr_conv_regs
                     );

  -- masks constant for Wishbone crossbar
  constant c_masks : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                  := (
                       c_slv_conv_regs => c_mask_conv_regs
                     );

  --============================================================================
  -- Component declarations
  --============================================================================
  -- Reset generator component
  -- (use: global reset generation, output reset generation)
  component reset_gen is
    generic
    (
      -- Reset time in number of clk_i cycles
      g_reset_time : positive := 5_000_000
    );
    port
    (
      clk_i   : in  std_logic;
      rst_i   : in  std_logic;
      rst_n_o : out std_logic
    );
  end component reset_gen;

  -- Pulse generator component
  -- (use: output pulse generation, pulse status LEDs)
  component ctb_pulse_gen is
    generic 
    (
      -- Pulse width, in number of clk_i cycles
      g_pulse_width     : natural := 15;

      -- Glitch filter length:
      -- g_glitch_filt_len=1 => trigger width should be > 1 clk_i cycle
      -- g_glitch_filt_len=2 => trigger width should be > 2 clk_i cycles
      -- etc.
      g_glitch_filt_len : natural := 4

    );
    port
    (
      clk_i        : in  std_logic;
      rst_n_i      : in  std_logic;

      -- Pulse type input
      -- '1' - TYPE 1 (glitch-sensitive, without output jitter)
      -- '0' - TYPE 2 (glitch-insensitive, with output jitter)
      pulse_type_i : in  std_logic;

      -- Enable input, pulse generation is enabled when '1'
      en_i         : in  std_logic;

      -- Trigger input, has to be '1' to assure pulse output with delay no greater
      -- than internal gate delays.
      trig_i       : in  std_logic;

      -- Pulse output, active-high
      -- latency:
      --    TYPE 1 pulse: none
      --    TYPE 2 pulse: g_glitch_filt_len+3 clk_i cycles
      pulse_o      : out std_logic
    );
  end component ctb_pulse_gen;

  -- RTM detector component
  -- (use: detect the presence of an RTM/P module)
  component rtm_detector is
    port 
    (
      rtmm_i    : in std_logic_vector(2 downto 0);
      rtmp_i    : in std_logic_vector(2 downto 0);
      rtmm_ok_o : out std_logic;
      rtmp_ok_o : out std_logic
    );
  end component rtm_detector;

  component ctb_pulse_gen_gp is
    generic
    (
      g_pwidth : natural := 200;
      g_freq   : natural := 400;
      g_delay  : natural := 0
    );
    port
    (
      clk_i   : in  std_logic;
      rst_n_i : in  std_logic;
      en_i    : in  std_logic;
      pulse_o : out std_logic
    );
  end component ctb_pulse_gen_gp;
  
  -- I2C bridge
  -- (use: convert I2C transfers into WB transfers on memmapped registers)  
  component elma_i2c is
    port
    (
      -- Clock, reset
      clk_i      : in  std_logic;
      rst_n_i    : in  std_logic;

      -- I2C lines
      sda_en_o   : out std_logic;
      sda_i      : in  std_logic;
      sda_o      : out std_logic;
      scl_en_o   : out std_logic;
      scl_i      : in  std_logic;
      scl_o      : out std_logic;

      -- I2C address and status
      i2c_addr_i : in  std_logic_vector(6 downto 0);
      i2c_done_o : out std_logic;
      i2c_err_o  : out std_logic;

      -- Wishbone master signals
      wbm_stb_o  : out std_logic;
      wbm_cyc_o  : out std_logic;
      wbm_sel_o  : out std_logic_vector(3 downto 0);
      wbm_we_o   : out std_logic;
      wbm_dat_i  : in  std_logic_vector(31 downto 0);
      wbm_dat_o  : out std_logic_vector(31 downto 0);
      wbm_adr_o  : out std_logic_vector(31 downto 0);
      wbm_ack_i  : in  std_logic;
      wbm_rty_i  : in  std_logic;
      wbm_err_i  : in  std_logic
    );
  end component elma_i2c;
  
  component conv_regs is
    port (
      rst_n_i                                  : in     std_logic;
      clk_sys_i                                : in     std_logic;
      wb_adr_i                                 : in     std_logic_vector(1 downto 0);
      wb_dat_i                                 : in     std_logic_vector(31 downto 0);
      wb_dat_o                                 : out    std_logic_vector(31 downto 0);
      wb_cyc_i                                 : in     std_logic;
      wb_sel_i                                 : in     std_logic_vector(3 downto 0);
      wb_stb_i                                 : in     std_logic;
      wb_we_i                                  : in     std_logic;
      wb_ack_o                                 : out    std_logic;
      wb_stall_o                               : out    std_logic;
    -- Port for std_logic_vector field: 'bits' in reg: 'Board ID register'
      conv_regs_id_bits_o                      : out    std_logic_vector(31 downto 0);
    -- Port for std_logic_vector field: 'fwvers' in reg: 'Status register'
      conv_regs_sr_fwvers_i                    : in     std_logic_vector(15 downto 0);
    -- Port for std_logic_vector field: 'switches' in reg: 'Status register'
      conv_regs_sr_switches_i                  : in     std_logic_vector(7 downto 0);
    -- Port for std_logic_vector field: 'RTM detection' in reg: 'Status register'
      conv_regs_sr_rtm_i                       : in     std_logic_vector(5 downto 0);
    -- Port for BIT field: 'Reset unlock bit' in reg: 'Control register'
      conv_regs_cr_rst_unlock_o                : out    std_logic;
    -- Port for BIT field: 'Reset bit' in reg: 'Control register'
      conv_regs_cr_rst_o                       : out    std_logic
    );
  end component conv_regs;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Clock signals
  signal clk125                    : std_logic;
  signal clk_50, clk_buf_50         : std_logic;
  signal clk_200, clk_buf_200       : std_logic;
  signal clk_250, clk_buf_250       : std_logic;
  signal pll_fb_in, pll_fb_out      : std_logic;
  signal pll_locked                 : std_logic;

  -- Reset signals
  signal rst_n, rst                 : std_logic;

  -- RTM detection signals
  signal rtmm, rtmp                 : std_logic_vector(2 downto 0);
  signal rtmm_ok, rtmp_ok           : std_logic;

  -- Signals to/from converter system registers component
  signal rtm_lines                  : std_logic_vector(5 downto 0);
  signal switches_n                 : std_logic_vector(7 downto 0);

  -- Signals for pulse generation triggers
  signal trig                       : std_logic_vector(g_nr_ttl_chan downto 1);
  signal trig_inv                   : std_logic_vector(g_nr_inv_chan downto 1);
  signal trig_ttl, trig_blo         : std_logic_vector(g_nr_ttl_chan downto 1);

  -- TTL-BAR lack of signal counter
  signal ttlbar_nosig_cnt           : t_ttlbar_nosig_cnt;
  signal ttlbar_nosig_n               : std_logic_vector(g_nr_ttl_chan downto 1); 

  -- Temporary signal for blocking and TTL pulse outputs
  signal pulse_outputs              : std_logic_vector(g_nr_ttl_chan downto 1);
  
  -- Temporary signal for inverted-TTL pulse outputs
  signal inv_outputs                : std_logic_vector(g_nr_inv_chan downto 1);
  
  -- Pulse status LED signals
  signal front_led_en               : std_logic_vector(g_nr_ttl_chan downto 1);
  signal rear_led_en                : std_logic_vector(g_nr_ttl_chan downto 1);
  signal pulse_leds                 : std_logic_vector(g_nr_ttl_chan downto 1);

  -- Output enable signals
  signal oe, ttl_oe, blo_oe, inv_oe : std_logic;

  -- Signal for controlling the bicolor LED matrix
  signal bicolor_led_state          : std_logic_vector(23 downto 0);

  signal tmp_pulse : std_logic;
  
  -- Wishbone crossbar signals
  signal xbar_slave_in       : t_wishbone_slave_in_array  (c_nr_masters - 1 downto 0);
  signal xbar_slave_out      : t_wishbone_slave_out_array (c_nr_masters - 1 downto 0);
  signal xbar_master_in      : t_wishbone_master_in_array (c_nr_slaves  - 1 downto 0);
  signal xbar_master_out     : t_wishbone_master_out_array(c_nr_slaves  - 1 downto 0);
  
  -- I2C bridge signals
  signal i2c_done            : std_logic;
  signal i2c_err             : std_logic;
  signal i2c_err_led         : std_logic;
  signal i2c_up              : std_logic;
  signal i2c_addr            : std_logic_vector(6 downto 0);
  signal led_i2c             : std_logic;
  signal led_i2c_clkdiv      : unsigned(22 downto 0);
  signal led_i2c_cnt         : unsigned( 2 downto 0);
  signal blink_state         : std_logic;
  
begin

  --============================================================================
  -- Generate 125 MHz global signal from differential lines
  --============================================================================
  cmp_125_diff_buf : IBUFGDS
  generic map 
  (
    DIFF_TERM    => TRUE,
    IBUF_LOW_PWR => TRUE
  )
  port map 
  (
    I  => fpga_clk_p_i,
    IB => fpga_clk_n_i,
    O  => clk125
  );

  --============================================================================
  -- Internal and external reset generation
  --============================================================================
  -- Configure reset generator for 96ms power-on reset
  cmp_reset_gen : reset_gen
    generic map
    (
      -- Reset time: 12 * 8ns * (10**6) = 96 ms
      g_reset_time => 12*(10**6)
    )
    port map
    (
      clk_i   => clk125,
      rst_i   => '0',
      rst_n_o => rst_n
    );
  
--  rst  <= not rst_n;
  mr_n_o <= rst_n;

  --============================================================================
  -- I2C bridge logic
  --============================================================================
  -- Set the I2C address signal according to ELMA protocol [1]
  i2c_addr <= "10" & fpga_ga_i;
  
  -- Instantiate I2C bridge component
  cmp_i2c_bridge : elma_i2c
    port map
    (
      -- Clock, reset
      clk_i      => clk125,
      rst_n_i    => rst_n,

      -- I2C lines
      sda_en_o   => sda_oe_o,
      sda_i      => sda_i,
      sda_o      => sda_o,
      scl_en_o   => scl_oe_o,
      scl_i      => scl_i,
      scl_o      => scl_o,

      -- I2C address and status
      i2c_addr_i => i2c_addr,
      i2c_done_o => i2c_done,
      i2c_err_o  => i2c_err,

      -- Wishbone master signals
      wbm_stb_o  => xbar_slave_in(0).stb,
      wbm_cyc_o  => xbar_slave_in(0).cyc,
      wbm_sel_o  => xbar_slave_in(0).sel,
      wbm_we_o   => xbar_slave_in(0).we,
      wbm_dat_i  => xbar_slave_out(0).dat,
      wbm_dat_o  => xbar_slave_in(0).dat,
      wbm_adr_o  => xbar_slave_in(0).adr,
      wbm_ack_i  => xbar_slave_out(0).ack,
      wbm_rty_i  => xbar_slave_out(0).rty,
      wbm_err_i  => xbar_slave_out(0).err
    );
  
  -- Process to blink the LED for a finite amount of time when the i2c_done
  -- signal is set.
  p_i2c_blink : process(clk125)
  begin
    if rising_edge(clk125) then
      if (rst_n = '0') then
        led_i2c_clkdiv <= (others => '0');
        led_i2c_cnt      <= (others => '0');
        led_i2c          <= '0';
        blink_state      <= '0';
      else
        case blink_state is

          when '0' =>
            led_i2c <= '0';
            if (i2c_done = '1') then
              blink_state <= '1';
            end if;

          when '1' =>
            led_i2c_clkdiv <= led_i2c_clkdiv + 1;
            if (led_i2c_clkdiv = 2499999) then
              led_i2c_clkdiv <= (others => '0');
              led_i2c_cnt    <= led_i2c_cnt + 1;
              led_i2c        <= not led_i2c;
              if (led_i2c_cnt = 7) then
                led_i2c_cnt <= (others => '0');
                blink_state <= '0';
              end if;
            end if;

          when others =>
            blink_state <= '0';

        end case;
      end if;
    end if;
  end process p_i2c_blink;

  -- Process to set the I2C error LED signal for display on the front panel
  -- of the front module. The I2C error signal is permanently set once an
  -- error is detected from the bridge module.
  p_i2c_err_led : process (clk125) is
  begin
  if rising_edge(clk125) then
    if (rst_n = '0') then
      i2c_err_led <= '0';
    elsif (i2c_err = '1') then
      i2c_err_led <= '1';
    end if;
  end if;
  end process p_i2c_err_led;

  --============================================================================
  -- Instantiation and connection of the main Wishbone crossbar
  --============================================================================
  xbar_master_in(0).int <= '0';
  xbar_master_in(0).err <= '0';

  cmp_wb_crossbar : xwb_crossbar
  generic map
  (
    g_num_masters => c_nr_masters,
    g_num_slaves  => c_nr_slaves,
    g_registered  => false,
    g_address     => c_addresses,
    g_mask        => c_masks
  )
  port map
  (
    clk_sys_i => clk125,
    rst_n_i   => rst_n,
    slave_i   => xbar_slave_in,
    slave_o   => xbar_slave_out,
    master_i  => xbar_master_in,
    master_o  => xbar_master_out
  );

  --============================================================================
  -- Converter board registers
  --============================================================================
  switches_n <= ttl_switch_n_i & extra_switch_n_i(7 downto 1);
  rtm_lines  <= rtmp & rtmm;

  cmp_conv_regs : conv_regs
    port map (
      rst_n_i                   => rst_n,
      clk_sys_i                 => clk125,
      wb_adr_i                  => xbar_master_out(c_slv_conv_regs).adr(3 downto 2),
      wb_dat_i                  => xbar_master_out(c_slv_conv_regs).dat,
      wb_dat_o                  => xbar_master_in (c_slv_conv_regs).dat,
      wb_cyc_i                  => xbar_master_out(c_slv_conv_regs).cyc,
      wb_sel_i                  => xbar_master_out(c_slv_conv_regs).sel,
      wb_stb_i                  => xbar_master_out(c_slv_conv_regs).stb,
      wb_we_i                   => xbar_master_out(c_slv_conv_regs).we,
      wb_ack_o                  => xbar_master_in (c_slv_conv_regs).ack,
      wb_stall_o                => xbar_master_in (c_slv_conv_regs).stall,

      conv_regs_id_bits_o       => open,
      conv_regs_sr_fwvers_i     => c_fwvers,
      conv_regs_sr_switches_i   => switches_n,
      conv_regs_sr_rtm_i        => rtm_lines,
      conv_regs_cr_rst_unlock_o => open,
      conv_regs_cr_rst_o        => open
    );

  --============================================================================
  -- Output enable logic
  --============================================================================
  -- The general output enable is set first and the blocking, TTL
  -- and INV output enable signals are set one clock cycle later.
  p_oe : process(clk125)
  begin
    if rising_edge(clk125) then
      if (rst_n = '0') then
        oe     <= '0';
        blo_oe <= '0';
        ttl_oe <= '0';
        inv_oe <= '0';
      else
        oe <= '1';
        if (oe = '1') then
          blo_oe <= '1';
          ttl_oe <= '1';
          inv_oe <= '1';
        end if;
      end if;
    end if;
  end process p_oe;

  fpga_oe_o          <= oe;
  fpga_blo_oe_o      <= blo_oe;
  fpga_trig_ttl_oe_o <= ttl_oe;
  fpga_inv_oe_o      <= inv_oe;

  --============================================================================
  -- TTL and blocking pulse generation logic
  --============================================================================
  -- First, the TTL trigger mux, selected via the TTL switch; ttlbar_nosig_n is
  -- controlled in the process below
  trig_ttl <= not fpga_input_ttl_n_i when (ttl_switch_n_i = '0') else 
              fpga_input_ttl_n_i and ttlbar_nosig_n;

  -- Then, the blocking trigger
  trig_blo <= fpga_blo_in_i;

  -- And now the OR gate at the inputs of the pulse generator blocks
  trig     <= trig_ttl or trig_blo;

  gen_ttl_pulse_generators : for i in 1 to g_nr_ttl_chan generate
    -- Process to detect lack of signal on TTL line
    --
    -- If the signal line is high for 100 us, the ttlbar_nosig_n lines disable
    -- the mux input.
    p_ttlbar_nosig : process(clk125)
    begin
      if rising_edge(clk125) then
        if (rst_n = '0') or (fpga_input_ttl_n_i(i) = '0') then
          ttlbar_nosig_n(i)   <= '1';
          ttlbar_nosig_cnt(i) <= (others => '0');
        elsif (fpga_input_ttl_n_i(i) = '1') then
          ttlbar_nosig_cnt(i) <= ttlbar_nosig_cnt(i) + 1;
          if (ttlbar_nosig_cnt(i) = 12499) then
            ttlbar_nosig_n(i)   <= '0';
            ttlbar_nosig_cnt(i) <= (others => '0');
          end if;
        end if;
      end if;
    end process p_ttlbar_nosig;

    -- Output pulse generators
    cmp_ttl_pulse_gen : ctb_pulse_gen
      generic map
      (
        g_pulse_width     => 150,
        g_glitch_filt_len => 4
      )
      port map
      (
        clk_i        => clk125,
        rst_n_i      => rst_n,
        en_i         => '1',
        pulse_type_i => extra_switch_n_i(1),
        trig_i       => trig(i),
        pulse_o      => pulse_outputs(i)
      );

    -- Status LED pulse generators
    cmp_led_pulse_gen : ctb_pulse_gen
      generic map
      (
        g_pulse_width     => 12*(10**6),
        g_glitch_filt_len => 4
      )
      port map
      (
        clk_i        => clk125,
        rst_n_i      => rst_n,
        en_i         => '1',
        pulse_type_i => extra_switch_n_i(1),
        trig_i       => trig(i),
        pulse_o      => pulse_leds(i)
      );
  end generate gen_ttl_pulse_generators;
    
  -- Pulse outputs assignment
  fpga_out_ttl_o  <= pulse_outputs when (ttl_switch_n_i = '0') else
                     not pulse_outputs;
  fpga_trig_blo_o <= pulse_outputs;
  
  -- Pulse status LED output assignments
  pulse_front_led_n_o <= (not pulse_leds) when (ttl_oe = '1') else 
                         (others => '0');
  pulse_rear_led_n_o  <= (not pulse_leds) when (blo_oe = '1') else 
                         (others => '0');

  --============================================================================
  -- General-purpose INV TTL outputs
  --============================================================================
  inv_out_o <= inv_in_n_i;
  
--  cmp_tmp_pulse_gen : ctb_pulse_gen_gp
--    generic map
--    (
--      g_pwidth => 100,
--      g_freq   => 125*(10**6)
--    )
--    port map
--    (
--      clk_i   => clk125,
--      rst_n_i => rst_n,
--      pulse_o => tmp_pulse
--    );
  
  --============================================================================
  -- Bicolor LED matrix logic
  --============================================================================
  -- Bicolor LED controls, corresponding to the column orders on the 
  -- bicolor_led_ctrl unit.

  -- WR address
  bicolor_led_state( 1 downto  0) <= c_LED_OFF;

  -- WR GMT
  bicolor_led_state( 3 downto  2) <= c_LED_OFF;

  -- WR link
  bicolor_led_state( 5 downto  4) <= c_LED_OFF;

  -- WR OK
  bicolor_led_state( 7 downto  6) <= c_LED_OFF;

  -- MULTICAST 0
  bicolor_led_state( 9 downto  8) <= c_LED_OFF;

  -- MULTICAST 1
  bicolor_led_state(11 downto 10) <= c_LED_OFF;


  -- I2C
  bicolor_led_state(13 downto 12) <= c_LED_GREEN when (led_i2c = '1') else
                                     c_LED_RED   when (i2c_err_led = '1') else
                                     c_LED_OFF;

  -- State of TTL/TTL_N switch
  bicolor_led_state(15 downto 14) <= c_LED_GREEN when (ttl_switch_n_i = '0') else
                                     c_LED_OFF;

  -- System error
  bicolor_led_state(17 downto 16) <= c_LED_RED when (rtmm_ok = '0') and (rtmp_ok = '0') else
                                     c_LED_OFF;

  -- System power
  bicolor_led_state(19 downto 18) <= c_LED_GREEN;

  -- MULTICAST 2
  bicolor_led_state(21 downto 20) <= c_LED_OFF; 

  -- MULTICAST 3
  bicolor_led_state(23 downto 22) <= c_LED_OFF;

  cmp_bicolor_led_ctrli : bicolor_led_ctrl
    generic map
    (
      g_NB_COLUMN    => 6,
      g_NB_LINE      => 2,
      g_clk_freq     => 125000000,
      g_refresh_rate => 250
    )
    port map
    (
      clk_i           => clk125,
      rst_n_i         => rst_n,
      led_intensity_i => "1111111",
      led_state_i     => bicolor_led_state,
      column_o(0)     => led_wr_ownaddr_i2c_o,
      column_o(1)     => led_wr_gmt_ttl_ttln_o,
      column_o(2)     => led_wr_link_syserror_o,
      column_o(3)     => led_wr_ok_syspw_o,
      column_o(4)     => led_multicast_2_0_o,
      column_o(5)     => led_multicast_3_1_o,
      line_o(0)       => led_ctrl0_o,
      line_o(1)       => led_ctrl1_o,
      line_oen_o(0)   => led_ctrl0_oen_o,
      line_oen_o(1)   => led_ctrl1_oen_o
    );

  --============================================================================
  -- RTM detection logic
  --============================================================================
  rtmm <= not fpga_rtmm_n_i;
  rtmp <= not fpga_rtmp_n_i;

  cmp_rtm_detector : rtm_detector
    port map
    (
      rtmm_i    => rtmm,
      rtmp_i    => rtmp,
      rtmm_ok_o => rtmm_ok,
      rtmp_ok_o => rtmp_ok
    );

end behav;
